"""
resultado:
    Info from http://fundamentus.com.br/resultado.php
"""


import fundamentus.utils as utils

import requests
import requests_cache
import pandas   as pd
import time, logging

from tabulate import tabulate


def get_resultado_raw():
    """
    Get data from fundamentus:
      URL:
        http://fundamentus.com.br/resultado.php

    RAW:
      DataFrame preserves original HTML header names

    Output:
      DataFrame
    """

    ##
    ## Busca avançada por empresa
    ##
    url = 'http://www.fundamentus.com.br/resultado.php'
    hdr = {'User-agent': 'Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:2.2) Gecko/20110201',
           'Accept': 'text/html, text/plain, text/css, text/sgml, */*;q=0.01',
           'Accept-Encoding': 'gzip, deflate',
           }

    with requests_cache.enabled():
        content = requests.get(url, headers=hdr)

        if content.from_cache:
            logging.debug('.../resultado.php: [CACHED]')
        else: # pragma: no cover
            logging.debug('.../resultado.php: sleeping...')
            time.sleep(.500) # 500 ms


    ## parse + load
    df = pd.read_html(content.text, decimal=",", thousands='.')[0]

    ## Fix: percent string
    df['Div.Yield']     = utils.perc_to_float( df['Div.Yield']     )
    df['Mrg Ebit']      = utils.perc_to_float( df['Mrg Ebit']      )
    df['Mrg. Líq.']     = utils.perc_to_float( df['Mrg. Líq.']     )
    df['ROIC']          = utils.perc_to_float( df['ROIC']          )
    df['ROE']           = utils.perc_to_float( df['ROE']           )
    df['Cresc. Rec.5a'] = utils.perc_to_float( df['Cresc. Rec.5a'] )

    ## index by 'Papel', instead of 'int'
    df.index = df['Papel']
    df.drop('Papel', axis='columns', inplace=True)
    df.sort_index(inplace=True)

    ## naming
    df.name = 'Fundamentus: HTML names'
    df.columns.name = 'Multiples'
    df.index.name = 'papel'

    ## return sorted by 'papel'
    return df


def get_resultado():
    """
    Data from fundamentus, fixing header names.
      URL:
        http://fundamentus.com.br/resultado.php
      Obs:
        DataFrame uses short header names
    Output:
      DataFrame
    """

    ## get RAW data
    data1 = get_resultado_raw()

    ## rename!
    data2 = _rename_cols(data1)

    ## metadata
    data2.name = 'Fundamentus: short names'
    data2.columns.name = 'Multiples'
    data2.index.name = 'papel'

    ## remove duplicates
#   df = data2.drop_duplicates(subset=['cotacao','pl','pvp'], keep='last')
    df = data2.drop_duplicates(keep='first')

    return df


def _rename_cols(data):
    """
    Rename columns in DataFrame
      - use a valid Python identifier
      - so each column can be a DataFrame property
      - Example:
          df.pl > 0
    """

    df2 = pd.DataFrame()

    ## Fix: rename columns
    df2['cotacao'  ] = data['Cotação'          ]
    df2['pl'       ] = data['P/L'              ]
    df2['pvp'      ] = data['P/VP'             ]
    df2['psr'      ] = data['PSR'              ]
    df2['dy'       ] = data['Div.Yield'        ]
    df2['pa'       ] = data['P/Ativo'          ]
    df2['pcg'      ] = data['P/Cap.Giro'       ]
    df2['pebit'    ] = data['P/EBIT'           ]
    df2['pacl'     ] = data['P/Ativ Circ.Liq'  ]
    df2['evebit'   ] = data['EV/EBIT'          ]
    df2['evebitda' ] = data['EV/EBITDA'        ]
    df2['mrgebit'  ] = data['Mrg Ebit'         ]
    df2['mrgliq'   ] = data['Mrg. Líq.'        ]
    df2['roic'     ] = data['ROIC'             ]
    df2['roe'      ] = data['ROE'              ]
    df2['liqc'     ] = data['Liq. Corr.'       ]
    df2['liq2m'    ] = data['Liq.2meses'       ]
    df2['patrliq'  ] = data['Patrim. Líq'      ]
    df2['divbpatr' ] = data['Dív.Brut/ Patrim.']
    df2['c5y'      ] = data['Cresc. Rec.5a'    ]

    return df2




#!/usr/bin/env python3
#
#

# import set_path_fundamentus

from fundamentus import get_resultado
from fundamentus import print_csv

if __name__ == '__main__':

    data = get_resultado()

    # Reorder by ticker
    data = data.sort_index(ascending=True)

    # filter on DataFrame
    data = data[ data.pl   > 0   ]
    data = data[ data.pl   < 100 ]
    data = data[ data.roe  > 0   ]
    data = data[ data.roic > 0   ]

    print_csv(data)





"""
Logging:
    init logging.
"""

import logging
import os


log_fmt='%(asctime)-15s [%(module)s.%(funcName)s] %(levelname)s: %(message)s'

def log_init():

    # default: info
    log_level = os.environ.get('LOGLEVEL') or 'INFO'

    logging.basicConfig(format=log_fmt, level=log_level.upper())

    logging.info('LOGLEVEL={}'.format(log_level))

    return


# logging.debug('This is a debug message')
# logging.info('This is an info message')
# logging.warning('This is a warning message')
# logging.error('This is an error message')
# logging.critical('This is a critical message')






"""
utils:
    Utility helpers.
"""

import requests
import requests_cache
import pandas   as pd
import logging

from tabulate import tabulate
from datetime import datetime
from dateutil.parser import parse


#
# Ref:
#   https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.replace.html
#   https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.Series.replace.html
#


def dt_iso8601(val):
    """
    Format dates to yyyy-mm-dd
    Input:
        scalar
    """
    try:
        dt = parse(val, dayfirst=True).strftime('%Y-%m-%d')
        return dt
    except: # pragma: no cover
        logging.debug('Error...')
        return



def from_pt_br(val):
    """
    from_pt_br: fix key/label by removing pt-br stuff
    Input:
        Series, i.e., a DataFrame column
    """
    res = val

    res.replace( to_replace=r'[?]'  , value=''  , regex=True, inplace=True )
    res.replace( to_replace=r'[(]'  , value=''  , regex=True, inplace=True )
    res.replace( to_replace=r'[)]'  , value=''  , regex=True, inplace=True )
    res.replace( to_replace=r'[$]'  , value=''  , regex=True, inplace=True )
    res.replace( to_replace=r'[.]'  , value=''  , regex=True, inplace=True )
    res.replace( to_replace=r'[/]'  , value=''  , regex=True, inplace=True )

    res.replace( to_replace=r'[ç]'  , value='c' , regex=True, inplace=True )
    res.replace( to_replace=r'[âáã]', value='a' , regex=True, inplace=True )
    res.replace( to_replace=r'[ôóõ]', value='o' , regex=True, inplace=True )
    res.replace( to_replace=r'[êé]' , value='e' , regex=True, inplace=True )
    res.replace( to_replace=r'[îí]' , value='i' , regex=True, inplace=True )
    res.replace( to_replace=r'[ûú]' , value='u' , regex=True, inplace=True )
    res.replace( to_replace=r'[ÛÚ]' , value='U' , regex=True, inplace=True )

    res.replace( to_replace=r'[ ]'  , value='_' , regex=True, inplace=True )
    res.replace( to_replace=r'__'   , value='_' , regex=True, inplace=True )

    return res


def fmt_dec(val):
    """
    Fix percent:
      - replace string in pt-br
      - from '45,56%' to '45.56%'

    Input:
        Series, i.e., a DataFrame column
    """

    res = val
    res = res.replace( to_replace=r'[.]', value='' , regex=True )
    res = res.replace( to_replace=r'[,]', value='.', regex=True )
#   res = res.astype(float)
#   res = res.astype(float) / 100
#   res = '{:4.2f}%'.format(res)

    return res


def perc_to_float(val):
    """
    Percent to float
      - replace string in pt-br to float
      - from '45,56%' to 0.4556

    Input:
        (DataFrame, column_name)
    """

    res = val
    res = res.replace( to_replace=r'[%]', value='' , regex=True )
    res = res.replace( to_replace=r'[.]', value='' , regex=True )
    res = res.replace( to_replace=r'[,]', value='.', regex=True )
    res = res.astype(float) / 100

    return res


def print_csv(data):
    """
    CSV printed to stdout
    """
    print(data.to_csv( index=True
                     , header=True
                     , decimal='.'
                     , float_format='%.4f'
                     )
         )

    return


def print_table(data):
    """
    Text table printed to stdout
      - separator: '|'
      - fixed-width columns for better reading
    """
    print( tabulate ( data
                    , headers=data.columns
                    , tablefmt='presto'
                    , floatfmt=".4f"
                    , disable_numparse=False
               )
     )

    return






"""
Fundamentus:
  Info:
    Get data from http://fundamentus.com.br/ using Pandas.

  Ref:
    https://github.com/mv/fundamentus
"""


__all__ = [
        'get_resultado',
        'get_resultado_raw',
        'get_papel',
        'list_papel_setor',
        ]

__version__ = '0.0.1'


from fundamentus.resultado  import get_resultado
from fundamentus.resultado  import get_resultado_raw

from fundamentus.detalhes   import get_papel
from fundamentus.detalhes   import get_detalhes_papel
from fundamentus.detalhes   import get_detalhes_raw
from fundamentus.detalhes   import list_papel_all

from fundamentus.setor      import get_setor_id
from fundamentus.setor      import list_papel_setor
from fundamentus.setor      import print_setores

from fundamentus.utils      import print_csv
from fundamentus.utils      import print_table


# log setup/init
import fundamentus
from   fundamentus import logging
fundamentus.logging.log_init()






from setuptools import setup
from setuptools import find_packages


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='fundamentus',
    packages=find_packages(where='src'),
    package_dir={'': 'src'},

    version='0.0.1',

    scripts=[
        'bin/fundamentus.csv.py',
        'bin/magic_formula.simple.py',
    ],

    url='https://github.com/mv/fundamentus/',
    author='Mv-Marcus Vinicius Ferreira',
    author_email='ferreira.mv@gmail.com',

    description='Fundamentus: API to load data from Fundamentus website: https://www.fundamentus.com.br/',
    long_description=long_description,
    long_description_content_type="text/markdown",


    install_requires=[
       'requests >= 2.25.1',
       'requests-cache >= 0.5.2',
       'pandas >= 1.1.5',
       'lxml >= 4.6.2',
       'tabulate >= 0.8.7',
    ],

)






"""
setor:
    Info from .../detalhes.php?setor=
"""

import requests
import requests_cache
import pandas   as pd
import time, logging

from   tabulate import tabulate


def list_papel_setor(setor=None):
    """
    Setor: ...

    Output:
      List
    """

    ## GET: setor
    url = 'http://www.fundamentus.com.br/resultado.php?setor={}'.format(setor)

    hdr = {'User-agent': 'Mozilla/5.0 (Windows; U; Windows NT 6.1; rv:2.2) Gecko/20110201' ,
           'Accept': 'text/html, text/plain, text/css, text/sgml, */*;q=0.01' ,
           'Accept-Encoding': 'gzip, deflate' ,
           }

    with requests_cache.enabled():
        content = requests.get(url, headers=hdr)

        if content.from_cache:
            logging.debug('.../resultado.php?setor={}: [CACHED]'.format(setor))
        else: # pragma: no cover
            logging.debug('.../resultado.php?setor={}: sleeping...'.format(setor))
            time.sleep(.500) # 500 ms


    ## parse + load
    df = pd.read_html(content.text, decimal=",", thousands='.')[0]

    ##
    return list(df['Papel'])


def get_setor_id(label):
    return df.T[label]['id']

def print_setores():
    print( tabulate(df, headers=['label','desc','id'], tablefmt='presto') )
    return


def _init_setor():
    data = pd.DataFrame(_setor, columns=['label','desc','id'])
    data.index = data['label']

    return data[['desc','id']]


## Setores:
_setor = [
   [ 'agro'            , 'Agropecuária'                       , 42 ] ,
   [ 'saneamento'      , 'Água e Saneamento'                  , 33 ] ,
   [ 'alimentos'       , 'Alimentos'                          , 15 ] ,
   [ 'bebidas'         , 'Bebidas'                            , 16 ] ,
   [ 'com1'            , 'Comércio'                           , 27 ] ,
   [ 'com2'            , 'Comércio'                           , 12 ] ,
   [ 'com3'            , 'Comércio e Distribuição'            , 20 ] ,
   [ 'computadores'    , 'Computadores e Equipamentos'        , 28 ] ,
   [ 'construcao'      , 'Construção e Engenharia'            , 13 ] ,
   [ 'engenharia'      , 'Construção e Engenharia'            , 13 ] ,
   [ 'diversos'        , 'Diversos'                           , 26 ] ,
   [ 'embalagens'      , 'Embalagens'                         , 6  ] ,
   [ 'energia'         , 'Energia Elétrica'                   , 32 ] ,
   [ 'equipamentos'    , 'Equipamentos Elétricos'             , 9  ] ,
   [ 'imoveis'         , 'Exploração de Imóveis'              , 39 ] ,
   [ 'financeiro'      , 'Financeiros'                        , 35 ] ,
   [ 'fumo'            , 'Fumo'                               , 17 ] ,
   [ 'gas'             , 'Gás'                                , 34 ] ,
   [ 'holdings'        , 'Holdings Diversificadas'            , 40 ] ,
   [ 'hoteis'          , 'Hoteis e Restaurantes'              , 24 ] ,
   [ 'restaurantes'    , 'Hoteis e Restaurantes'              , 24 ] ,
   [ 'papel'           , 'Madeira e Papel'                    , 5  ] ,
   [ 'maquinas'        , 'Máquinas e Equipamentos'            , 10 ] ,
   [ 'materiais'       , 'Materiais Diversos'                 , 7  ] ,
   [ 'transporte'      , 'Material de Transporte'             , 8  ] ,
   [ 'midia'           , 'Mídia'                              , 23 ] ,
   [ 'mineracao'       , 'Mineração'                          , 2  ] ,
   [ 'outros'          , 'Outros'                             , 41 ] ,
   [ 'petroleo'        , 'Petróleo, Gás e Biocombustíveis'    , 1  ] ,
   [ 'previdencia'     , 'Previdência e Seguros'              , 38 ] ,
   [ 'seguros'         , 'Previdência e Seguros'              , 38 ] ,
   [ 'usopessoal'      , 'Prods. de Uso Pessoal e de Limpeza' , 18 ] ,
   [ 'limpeza'         , 'Prods. de Uso Pessoal e de Limpeza' , 18 ] ,
   [ 'programas'       , 'Programas e Serviços'               , 29 ] ,
   [ 'quimicos'        , 'Químicos'                           , 4  ] ,
   [ 'saude'           , 'Saúde'                              , 19 ] ,
   [ 'securitizadoras' , 'Securitizadoras de Recebíveis'      , 36 ] ,
   [ 'servicos'        , 'Serviços'                           , 11 ] ,
   [ 'finandiversos'   , 'Serviços Financeiros Diversos'      , 37 ] ,
   [ 'siderurgia'      , 'Siderurgia e Metalurgia'            , 3  ] ,
   [ 'tecidos'         , 'Tecidos, Vestuário e Calçados'      , 21 ] ,
   [ 'vestuario'       , 'Tecidos, Vestuário e Calçados'      , 21 ] ,
   [ 'telecom'         , 'Telecomunicações'                   , 43 ] ,
   [ 'telefoniafixa'   , 'Telefonia Fixa'                     , 30 ] ,
   [ 'telefoniamovel'  , 'Telefonia Móvel'                    , 31 ] ,
   [ 'transporte'      , 'Transporte'                         , 14 ] ,
   [ 'utilidades'      , 'Utilidades Domésticas'              , 22 ] ,
   [ 'viagens'         , 'Viagens e Lazer'                    , 25 ] ,
]
##

# setup/load
df = _init_setor()


